/*
** config_scan.c - part of GrabURL 2.0
** Copyright (C) 1996-97 Serge Emond
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "regex.h"
#include "libgurl.h"
#include "macros.h"
#include "config.h"
#include "config_scan.h"

/* Add: check for duplicates */

void cfg_scan_init(void)
{
	memset(&scan_cfg, 0, sizeof(struct Scan_Cfg));
	scan_cfg.flags = SCAN_DIESE_SEARCH;
}

char *cfg_cmd_key(const char *line)
{
	char *arg;
	struct Scan_Key *k;

	arg = ExtractArgument(line, 1);
	if (!arg) return "Missing argument to Key";

	k = (struct Scan_Key *)gmalloc(sizeof(struct Scan_Key));
	k->next = scan_cfg.key;
	k->key = arg;
	scan_cfg.key = k;

	return 0;
}

char *cfg_cmd_ignore(const char *line)
{
	struct Scan_Ignore *p;

	p = (struct Scan_Ignore *)gmalloc(sizeof(struct Scan_Ignore));
	p->beg = ExtractArgument(line, 1);
	p->end = ExtractArgument(line, 2);
	if (!p->beg) return "Missing argument to Ignore";

	p->next = scan_cfg.ignore;
	scan_cfg.ignore = p;
	return 0;
}

char *cfg_cmd_diese(const char *line)
{
	char *arg;

	arg = ExtractArgument(line, 1);
	if (!arg) return "Missing argument to Diese";

	if (!str_ieq(arg, "search")) scan_cfg.flags |= SCAN_DIESE_SEARCH;
	else if (!str_ieq(arg, "normal")) scan_cfg.flags &= ~SCAN_DIESE_SEARCH;
	else {
	  free(arg);
	  return "Bad argument to Diese";
	}
	free(arg);
	return 0;
}

char *cfg_cmd_skip(const char *line)
{
	char *arg;
	struct Pat_List *p;

	arg = ExtractArgument(line, 1);
	if (!arg) {
	  return "Missing argument to Skip";
	}
	p = (struct Pat_List *)gmalloc(sizeof(struct Pat_List));
	free(arg);
	if (!regcomp(&p->pat, arg, REG_ICASE | REG_NOSUB)) {
	  free(p);
	  return "Can't compile the skip regexp";
	}
	p->next = scan_cfg.skip;
	scan_cfg.skip = p;
	return 0;
}

char *cfg_cmd_pattern(const char *line)
{
	char *arg;
	struct Pat_List *p;

	arg = ExtractArgument(line, 1);
	if (!arg) {
	  return "Missing argument to Pattern";
	}
	p = (struct Pat_List *)gmalloc(sizeof(struct Pat_List));
	free(arg);
	if (!regcomp(&p->pat, arg, REG_ICASE | REG_NOSUB)) {
	  free(p);
	  return "Can't compile pattern regexp";
	}
	p->next = scan_cfg.accept;
	scan_cfg.accept = p;
	return 0;
}

int rargs_scan(char **argv, int pos)
{
	int delta;

	if ((str_ieq(argv[pos], "p") == 0) ||
			 (str_ieq(argv[pos], "pattern") == 0)) {
	  struct Pat_List *pl;

	  pl = (struct Pat_List *)gmalloc(sizeof(struct Pat_List));
	  pl->next = scan_cfg.accept;
	  scan_cfg.accept = pl;
	  if (regcomp(&pl->pat, argv[++pos], REG_ICASE | REG_NOSUB)) {
		fprintf(stderr, "Bad regexp pattern \"%s\"\n", argv[pos]);
		exit(EXIT_FAILURE);
	  }
	  delta = 2;
	}
	else if ((str_ieq(argv[pos], "sp") == 0) ||
			 (str_ieq(argv[pos], "skippattern") == 0)) {
	  struct Pat_List *pl;
	  pl = (struct Pat_List *)gmalloc(sizeof(struct Pat_List));
	  pl->next = scan_cfg.skip;
	  scan_cfg.skip = pl;
	  if (regcomp(&pl->pat, argv[++pos], REG_ICASE | REG_NOSUB)) {
		fprintf(stderr, "Bad regexp skip \"%s\"\n", argv[pos]);
		exit(EXIT_FAILURE);
	  }
	  delta = 2;
	}
	else
	  delta = 0;

	return(delta);
}

